
from __future__ import absolute_import
from collections import namedtuple

from .util import clean_id

"""
Functions for working with Brazilian zipcodes.

"""


CEP = namedtuple('CEP', ['cep', 'region', 'subregion', 'sector', 'subsector',
                         'division', 'suffix'])


def format_cep(cep):
    """Applies typical 00000-000 formatting to CEP."""
    cep = clean_id(cep)
    dig = len(cep)

    if dig == 4 or dig == 5:
        cep = '0'*(5-dig) + cep + '000'
    elif dig == 7 or dig == 8:
        cep = '0'*(8-dig) + cep
    else:
        raise ValueError('Invalid CEP code: {0}'.format(cep))

    return '{0}-{1}'.format(cep[:-3], cep[-3:])


def parse_cep(cep, numeric=True):
    """Split CEP into region, sub-region, sector, subsector, division."""
    fmtcep = format_cep(cep)
    if numeric:
        cep = int(fmtcep.replace('-', ''))
        geo = [int(fmtcep[:i]) for i in range(1, 6)]
        suffix = int(fmtcep[-3:])
    else:
        cep = fmtcep
        geo = [fmtcep[:i] for i in range(1, 6)]
        suffix = fmtcep[-3:]

    return CEP(cep, geo[0], geo[1], geo[2], geo[3], geo[4], suffix)



#!/usr/bin/env python
# -*- coding: utf8 -*-

from __future__ import absolute_import

from .util import clean_id

"""
Functions for working with Brazilian municipality (municipio) codes.

"""

MUNI_WEIGHTS = [1, 2, 1, 2, 1, 2]

# there are 9 IBGE municipal codes with invalid check digits;
# see, http://www.sefaz.al.gov.br/nfe/notas_tecnicas/NT2008.004.pdf
SHIM = {
    '2201919': 9,  # Bom Princípio do Piauí, PI
    '2201988': 8,  # Brejo do Piauí, PI
    '2202251': 1,  # Canavieira, PI
    '2611533': 3,  # Quixaba, PE
    '3117836': 6,  # Cônego Marinho, MG
    '3152131': 1,  # Ponto Chique, MG
    '4305871': 1,  # Coronel Barros, RS
    '5203939': 9,  # Buriti de Goiás, GO
    '5203962': 2,  # Buritinópolis, GO
}


def validate_muni(muni):
    """Check whether municipio code is valid."""
    muni = clean_id(muni)
    # municipal codes are 7 digits long, and cannot start with 0
    if len(muni) != 7:
        return False
    digits = [int(k) for k in muni]
    valid = _muni_check(digits[:-1]) == digits[-1]
    return valid or muni in SHIM  # need to check exceptions list


def muni_check_digit(muni):
    """Find check digit needed to make a valid municipio code."""
    muni = clean_id(muni)
    if len(muni) < 6:
        raise ValueError('Municipio must have at least 6 digits: {0}'
                         .format(muni))
    if muni in SHIM:
        return SHIM[muni]

    digits = [int(k) for k in muni[:7]]
    return _muni_check(digits)


def _muni_check(digits):
    """Calculate check digit from iterable of integers."""
    digmul = (w * k for w, k in zip(MUNI_WEIGHTS, digits))
    digsum = sum(n if n < 10 else 1 + (n % 10) for n in digmul)
    modulo = digsum % 10
    return 0 if modulo == 0 else 10 - modulo



#!/usr/bin/env python

from __future__ import absolute_import

import re
from random import randint

from .util import clean_id, pad_id

"""
Functions for working with Brazilian PIS/PASEP identifiers.

"""

NONDIGIT = re.compile(r'[^0-9]')
PIS_WEIGHTS = [3, 2, 9, 8, 7, 6, 5, 4, 3, 2]


def validate_pis(pis, autopad=True):
    """Check whether PIS/PASEP is valid. Optionally pad if too short."""
    pis = clean_id(pis)
    # all complete PIS/PASEP are 11 digits long
    if len(pis) != 11:
        return validate_pis(pad_pis(pis), False) if autopad else False
    return int(pis[-1]) == _pis_check(pis)


def pis_check_digit(pis):
    """Find check digit needed to make a PIS/PASEP valid."""
    pis = clean_id(pis)
    if len(pis) < 10:
        raise ValueError(
            'PIS/PASEP must be at least 10 digits: {0}'.format(pis))
    return _pis_check(pis)


def pis_check_digits(pis):
    """Alias for pis_check_digit function. PIS/PASEP uses single digit."""
    return pis_check_digit(pis)


def format_pis(pis):
    """Applies typical 000.0000.000-0 formatting to PIS/PASEP."""
    pis = pad_pis(pis)
    fmt = '{0}.{1}.{2}-{3}'
    return fmt.format(pis[:3], pis[3:7], pis[7:10], pis[10])


def pad_pis(pis, validate=False):
    """Takes a PIS/PASEP that had leading zeros and pads it."""
    padded = pad_id(pis, '%0.011i')
    if validate:
        return padded, validate_pis(padded)
    return padded


def random_pis(formatted=True):
    """Create a random, valid PIS identifier."""
    pis = randint(1000000000, 9999999999)
    pis = str(pis) + str(pis_check_digit(pis))
    if formatted:
        return format_pis(pis)
    return pis


def _pis_check(pis):
    """Calculate check digit from string."""
    digits = [int(k) for k in pis[:11]]
    # find check digit
    cs = sum(w * k for w, k in zip(PIS_WEIGHTS, digits)) % 11
    return 0 if cs < 2 else 11 - cs




import re

"""
Helper functions for validating identifiers.

"""

NONDIGIT = re.compile(r'[^0-9]')


def clean_id(identifier):
    """Remove non-numeric characters from input."""
    if isinstance(identifier, int):
        return str(identifier)
    return NONDIGIT.sub('', identifier)


def pad_id(identifier, fmt):
    """Pad an identifier with leading zeros."""
    if not isinstance(identifier, int):
        identifier = int(clean_id(identifier))
    return fmt % identifier



# -*- coding: utf8 -*-

from distutils.core import setup

setup(
    name='brazilnum',
    packages=['brazilnum'],
    version='0.8.6',
    description="Validate Brazilian CNPJ, CEI, CPF, PIS/PASEP, CEP, and municipal numbers",
    author='Chris Poliquin',
    author_email='cpoliquin@hbs.edu',
    url='https://github.com/poliquin/brazilnum',
    keywords=['brazil', 'brasil', 'cnpj', 'cei', 'cpf', 'pis', 'pasep', 'cep'],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Utilities'
        ],
    long_description="""\
Validate and Parse Brazilian Identification Numbers
---------------------------------------------------

Python functions for working with CNPJ, CEI, CPF, PIS/PASEP, CEP, and municipal
(município) codes, which identify firms, people, and places in Brazil.
"""
)
